#ifndef ISOELECTRICPOINT_H
#define ISOELECTRICPOINT_H
 
#include "ui_isoelectricpoint.h"
 
 
class IEPapp : public QWidget, private Ui::IsoelectricPoint
{
    Q_OBJECT
 
public:
    IEPapp(QWidget *parent = 0);
 
 
public slots:

    void getPath();
    void about();
    void calculate();
    void save();

};
 
 
#endif
