#include <QApplication>

#include "isoelectricpoint.h"

int main(int argc, char *argv[])
{
    QApplication app(argc, argv);
    IEPapp *dialog = new IEPapp;

    dialog->show();
    return app.exec();
}

